/*
 * Kosmos.
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package hu.midori.kosmos.server;

import java.io.Serializable;

import net.sf.ehcache.Cache;
import net.sf.ehcache.Element;

import org.aopalliance.intercept.MethodInterceptor;
import org.aopalliance.intercept.MethodInvocation;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Intercepts the service method calls and caches the result.
 * <p>
 * Implementation is based on:
 * http://opensource2.atlassian.com/confluence/spring/display/DISC/Caching+the+result+of+methods+using+Spring+and+EHCache
 *
 * @author <a href="mailto:aron.gombas@midori.hu">Aron Gombas</a>
 * @version $Id$
 */
public class MethodResultCacheInterceptor implements MethodInterceptor {
    private final static Log log = LogFactory.getLog(MethodResultCacheInterceptor.class);

    /** <i>EHCache</i> implementation. */
    private Cache cache;

    public void setCache(Cache cache) {
        this.cache = cache;
    }

    /** Caches the return value of the intercepted method. */
    public Object invoke(MethodInvocation invocation) throws Throwable {
        log.debug(String.format("Accessing cache \"%s\" [%d items]...", cache.getName(), cache.getSize()));

        // look-up
        String targetName = invocation.getThis().getClass().getName();
        String methodName = invocation.getMethod().getName();
        Object[] arguments = invocation.getArguments();
        String key = getCacheKey(targetName, methodName, arguments);
        Element element = cache.get(key);
        if(element != null) {
            log.info(String.format("Cache-hit: \"%s\" returned from cache", key));
            return element.getValue();
        }

        // call target if not found
        log.info(String.format("Cache-miss: reloading \"%s\"...", key));
        Serializable value = (Serializable)invocation.proceed();
        if(value != null) {
            cache.put(new Element(key, value));
            log.debug(String.format("\"%s\" saved to cache", key));
        }

        return value;
    }

    /**
     * Returns the cache key in the format
     * <code>targetName.methodName.argument0.argument1 ...</code>.
     */
    private String getCacheKey(String targetName, String methodName, Object[] arguments) {
        StringBuffer buffer = new StringBuffer();

        buffer.append(targetName).append(".").append(methodName);
        if ((arguments != null) && (arguments.length != 0)) {
            for (int i=0; i<arguments.length; i++)
                buffer.append(".").append(arguments[i]);
        }

        return buffer.toString();
    }
}
