/*
 * Kosmos.
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package hu.midori.kosmos.server.util;

import java.awt.Color;
import java.awt.Rectangle;
import java.awt.Shape;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Map;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartUtilities;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.PiePlot;
import org.jfree.chart.renderer.xy.XYBarRenderer;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.general.DefaultPieDataset;
import org.jfree.data.general.PieDataset;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;

/**
 * Utility methods for chart generation.
 *
 * @author <a href="mailto:aron.gombas@midori.hu">Aron Gombas</a>
 * @version $Id$
 */
public class ChartUtils {
    /**
     * Max number of items in trimmed datasets.
     * @see #collectionToPieDataset(Collection, int)
     */
    public static final int MAX_ITEMS = 8;

    /** Transparent paint used as background in chart images. */
    protected static final Color TRANSPARENT_COLOR = new Color(0, 0, 0, 0);
    /** Shape for the timeseries-charts. */
    protected static final Shape RECTANGLE_SHAPE = new Rectangle(-1, -1, 2, 2);

    /** This class should never be instantiated. */
    protected ChartUtils() { }

    /** Returns the pie-chart generated from the passed data. */
    public static JFreeChart generatePieChart(PieDataset dataset) {
        // generate chart
        JFreeChart chart = ChartFactory.createPieChart(null, dataset, false, false, false);
        chart.setBackgroundPaint(TRANSPARENT_COLOR);

        return chart;
    }

    /**
     * Returns the color-coded pie-chart generated from the passed data.
     * The keys in the dataset are <code>ColorCodedKey</code> objects that
     * specify both the pie section labels and the pie section colors.
     */
    public static JFreeChart generateColorCodedPieChart(PieDataset dataset) {
        // generate chart
        JFreeChart chart = generatePieChart(dataset);

        // apply color-codes
        PiePlot plot = (PiePlot)chart.getPlot();
        for(int i = 0; i < dataset.getItemCount(); i++)
            plot.setSectionPaint(i, ((ColorCodedKey)dataset.getKey(i)).getColor());

        return chart;
    }

    /** Returns the time-chart generated from the passed data. */
    public static JFreeChart generateTimeLineChart(TimeSeries dataset) {
        // generate chart
        JFreeChart chart = ChartFactory.createTimeSeriesChart(null, null, null, new TimeSeriesCollection(dataset), false, false, false);
        chart.setBackgroundPaint(TRANSPARENT_COLOR);

        // override renderer
        XYLineAndShapeRenderer renderer = (XYLineAndShapeRenderer)(chart.getXYPlot().getRenderer());
        renderer.setShapesVisible(true);
        renderer.setShape(RECTANGLE_SHAPE);

        return chart;
    }

    /** Returns the time-barchart generated from the passed data. */
    public static JFreeChart generateTimeBarChart(TimeSeries dataset) {
        // generate chart
        JFreeChart chart = generateTimeLineChart(dataset);

        // override renderer with bar type
        XYBarRenderer renderer = new XYBarRenderer(0.2);
        renderer.setDrawBarOutline(false);
        chart.getXYPlot().setRenderer(renderer);

        return chart;
    }

    /** Writes the given chart to the output-stream in PNG image format. */
    public static void writeChartAsPng(JFreeChart chart, OutputStream out, int width, int height) throws IOException {
        ChartUtilities.writeChartAsPNG(out, chart, width, height, true, 0);
    }

    /**
     * Returns the passed collection as untrimmed pie-dataset.
     * @see #collectionToPieDataset(Collection, int)
     */
    public static PieDataset collectionToPieDataset(Collection items) {
        return collectionToPieDataset(items, -1);
    }

    /**
     * Returns the passed collection of <code>Map.Entry</code> objects as pie-dataset
     * (equivalent key-value pairs), optionally trimmed to a maximum number of items
     * to prevent unreadably busy charts.
     * @param maxItems the maximum number of items in the returned pie-dataset or -1 if trimming is not needed.
     */
    @SuppressWarnings("unchecked")
    public static PieDataset collectionToPieDataset(Collection items, int maxItems) {
        DefaultPieDataset dataset = new DefaultPieDataset();

        // generate dataset
        int counter = 0;
        for(Map.Entry<? extends Comparable, Integer> item : (Collection<Map.Entry<? extends Comparable, Integer>>)items) {
            if((maxItems != -1) && (++counter > maxItems))
                break;
            dataset.setValue(item.getKey(), item.getValue());
        }

        return dataset;
    }

    /**
     * Increments the counter (or starts that from 1 if not existing yet) in
     * a <code>String</code> to <code>Integer</code> map.
     */
    public static void incrementCountMap(Map<String, Integer> map, String key) {//TODO move to Chartutils
        if(key == null)
            return;

        Integer count = map.get(key);
        map.put(key, (count == null) ? 1 : count + 1);
    }

    /** Returns the list of the entries in the map sorted by their value (not by their key!) in descending order. */
    @SuppressWarnings("unchecked")
    public static List countMapToSortedList(Map map) {// TODO should be Map<?, Integer> and List<Map.Entry<?, Integer>>
        List<Map.Entry> sorted = new ArrayList<Map.Entry>(map.entrySet());
        Collections.sort(sorted, new Comparator() {// TODO should be Comparator<Map.Entry<?, Integer>>
            public int compare(Object obj1, Object obj2) {
                // revert-compare for descending order
                Integer value1 = (Integer)((Map.Entry)obj1).getValue();
                Integer value2 = (Integer)((Map.Entry)obj2).getValue();
                int comparison = value2.compareTo(value1);
                if(comparison != 0)
                    return comparison;

                // compare keys in ascending order for identical-valued entries
                String key1 = (String)((Map.Entry)obj1).getKey();
                String key2 = (String)((Map.Entry)obj2).getKey();
                return key1.compareTo(key2);
            }
        });

        return sorted;
    }
}
